#include <boost/graph/use_mpi.hpp>
#include <boost/graph/distributed/mpi_process_group.hpp>
#include <boost/graph/distributed/adjacency_list.hpp>
#include <boost/graph/connected_components.hpp>

int 
main(int argc, char* argv[])
{
	using namespace std;
	using namespace boost;

	typedef adjacency_list<listS, distributedS<boost::graph::distributed::mpi_process_group, vecS>, undirectedS> graph;
	typedef property_map<graph, vertex_index_t>::const_type index_map;
	typedef graph_traits<graph>::vertex_descriptor vertex;
	typedef graph_traits<graph>::vertex_iterator iterator;
	typedef graph_traits<graph>::adjacency_iterator adjacency_iterator;
	typedef pair<iterator, iterator> iterators;
	typedef pair<adjacency_iterator, adjacency_iterator> adjacency_iterators;
	typedef vector<vertex> vertex_list;
	typedef vector<vertex_list>::size_type size_type;

	const mpi::environment env(argc, argv);
	const mpi::communicator world;
	graph clusters;

	const int rank = world.rank();
	const char filename[] = {'a'+rank,'\0'};
	ifstream in(filename);
	
	graph_traits<graph>::degree_size_type num_neighbors;
	vector<vertex_list> neighbors;
	vector<size_type> sizes;

	// read neighbors
	for (in >> num_neighbors; in; in >> num_neighbors) 
		for (neighbors.push_back (vertex_list()); num_neighbors; --num_neighbors)
		{
			processor_id_type owner;
			vertex::local_descriptor_type local;
			in >> owner >> local;
			neighbors.back ().push_back (vertex(owner, local));
		}
	mpi::all_gather (world, neighbors.size (), sizes);

	// add vertices
	for (size_type i = 0; neighbors.size () > i; ++i) add_vertex (clusters);

	// add edges
	for (size_type i = 0; neighbors.size () > i; ++i)
		for (vertex_list::const_iterator j = neighbors[i].begin(); neighbors[i].end() != j; ++j)
	{
			assert (local(*j) < sizes[owner(*j)]);
			add_edge (vertex(rank, i), *j, clusters);
	}
	synchronize (clusters.process_group ());

	// create parent map
	vertex* const parents = new vertex[num_vertices(clusters)];
	iterator_property_map<vertex*, index_map> parent_map (parents, get(vertex_index, clusters));
	
	// print neighbors of problem vertex
	const vertex problem (0, 20);
	if (!rank) for (adjacency_iterators i = adjacent_vertices (problem, clusters); i.first != i.second; ++i.first)
		cout << "neighbor of (0,20): (" << owner (*i.first) << "," << local (*i.first) << ")" << std::endl;

	// compute connected components
	boost::graph::connected_components (clusters, dummy_property_map(), parent_map);

	// print parent of problem vertex
	request (parent_map, problem);
	synchronize (parent_map);
	vertex parent = get (parent_map, problem);
	if (!rank) cout << "parent of (0,20): (" << owner (parent) << "," << local (parent) << ")" << std::endl;

	// print parent's children
	for (iterators i = vertices (clusters); i.first != i.second; ++i.first) 
		if (get (parent_map, *i.first) == parent)
			cout << "sibling of (0,20): (" << owner (*i.first) << "," << local (*i.first) << ")" << std::endl;

	// free memory
	delete[] parents;
	return EXIT_SUCCESS;
}
